﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;

namespace ProjektStatki
{
    public abstract class Enemy : Character
    {
        protected Vector2 direction;
        protected Location pathToPlayer;
        protected Location currentLocation;
        protected Location nextLocation;

        public class Location
        {
            public int X;
            public int Y;
            public int F;
            public int G;
            public int H;
            public Location Parent;
        }

        protected void Navigate()
        {
            var start = new Location { X = this.xTile, Y = this.yTile };
            var target = new Location { X = Game1.Player.xTile, Y = Game1.Player.yTile };
            currentLocation = start;
            Debug.WriteLine(start.X);
            Debug.WriteLine(start.Y);
            Debug.WriteLine(target.X);
            Debug.WriteLine(target.Y);

            // algorithm  
            Location current = null;
            var openList = new List<Location>();
            var closedList = new List<Location>();
            int g = 0;

            // start by adding the original position to the open list  
            openList.Add(start);

            while (openList.Count > 0)
            {
                // get the square with the lowest F score  
                var lowest = openList.Min(l => l.F);
                current = openList.First(l => l.F == lowest);

                // add the current square to the closed list  
                closedList.Add(current);

                // remove it from the open list  
                openList.Remove(current);

                // if we added the destination to the closed list, we've found a path  
                if (closedList.FirstOrDefault(l => l.X == target.X && l.Y == target.Y) != null) { break; }

                var adjacentSquares = GetWalkableAdjacentSquares(current.X, current.Y, openList);
                g = current.G + 1;

                foreach (var adjacentSquare in adjacentSquares)
                {
                    // if this adjacent square is already in the closed list, ignore it  
                    if (closedList.FirstOrDefault(l => l.X == adjacentSquare.X
                        && l.Y == adjacentSquare.Y) != null)
                        continue;

                    // if it's not in the open list...  
                    if (openList.FirstOrDefault(l => l.X == adjacentSquare.X
                        && l.Y == adjacentSquare.Y) == null)
                    {
                        // compute its score, set the parent  
                        adjacentSquare.G = g;
                        adjacentSquare.H = ComputeHScore(adjacentSquare.X, adjacentSquare.Y, target.X, target.Y);
                        adjacentSquare.F = adjacentSquare.G + adjacentSquare.H;
                        adjacentSquare.Parent = current;

                        // and add it to the open list  
                        openList.Insert(0, adjacentSquare);
                    }
                    else
                    {
                        // test if using the current G score makes the adjacent square's F score  
                        // lower, if yes update the parent because it means it's a better path  
                        if (g + adjacentSquare.H < adjacentSquare.F)
                        {
                            adjacentSquare.G = g;
                            adjacentSquare.F = adjacentSquare.G + adjacentSquare.H;
                            adjacentSquare.Parent = current;
                        }
                    }
                }
            }

            pathToPlayer = current;
        }

        static List<Location> GetWalkableAdjacentSquares(int x, int y, List<Location> openList)
        {
            List<Location> list = new List<Location>();
            int cols = Game1.Level.cols;
            int rows = Game1.Level.rows;

            if ( y-1 > 0 && !Game1.Level.Tiles[(y-1)*cols + x].collidable )
            {
                Location node = openList.Find(l => l.X == x && l.Y == y - 1);
                if (node == null) list.Add(new Location() { X = x, Y = y - 1 });
                else list.Add(node);
            }

            if ( y+1 < cols && !Game1.Level.Tiles[(y + 1) * cols + x].collidable)
            {
                Location node = openList.Find(l => l.X == x && l.Y == y + 1);
                if (node == null) list.Add(new Location() { X = x, Y = y + 1 });
                else list.Add(node);
            }

            if ( x-1 > 0 && !Game1.Level.Tiles[y * cols + x - 1].collidable)
            {
                Location node = openList.Find(l => l.X == x - 1 && l.Y == y);
                if (node == null) list.Add(new Location() { X = x - 1, Y = y });
                else list.Add(node);
            }

            if ( x+1 < rows && !Game1.Level.Tiles[y * cols + x + 1].collidable)
            {
                Location node = openList.Find(l => l.X == x + 1 && l.Y == y);
                if (node == null) list.Add(new Location() { X = x + 1, Y = y });
                else list.Add(node);
            }

            return list;
        }

        static int ComputeHScore(int x, int y, int targetX, int targetY)
        {
            return Math.Abs(targetX - x) + Math.Abs(targetY - y);
        }

        protected void GetNextLocation()
        {
            Location tmp = pathToPlayer;
            while (tmp.Parent != null &&tmp.Parent != currentLocation) { tmp = tmp.Parent; }
            nextLocation = tmp;
        }
    }
}
