﻿using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace ProjektStatki
{
    /* This class represents all drawable objects 
     * Each object of this type needs to be added to game objest list upon creation
     */
    public abstract class GameObject
    {
        protected Rectangle hitbox;
        public Rectangle Hitbox { get => hitbox; }
        protected Texture2D bitmap;
        protected Color[] bitmapData;
        protected float rotation = 0;
        public float Rotation { get => this.rotation; }
        public int x { get => this.hitbox.left; }
        public int y { get => this.hitbox.top; }
        public int width { get => this.hitbox.width; }
        public int height { get => this.hitbox.height; }
        protected double xPos;
        protected double yPos;
        public Vector2 Pos { get => new Vector2((float)xPos, (float)yPos);  }
        protected Vector2 origin;
        public Vector2 Origin { get => this.origin; }
        public Matrix Transform
        {
            get
            {
                // There is no overload for 2D vector
                return Matrix.CreateTranslation(new Vector3(-Origin, 0)) *
                       Matrix.CreateRotationZ(Rotation) *
                       Matrix.CreateTranslation(new Vector3((float)x, (float)y, 0));
            }
        }

        public bool Overlaps(GameObject obj)
        {
            this.UpdateOrigin();
            obj.UpdateOrigin();

            Matrix transformToObj = this.Transform * Matrix.Invert(obj.Transform);

            Vector2 stepX = Vector2.TransformNormal(Vector2.UnitX, transformToObj);
            Vector2 stepY = Vector2.TransformNormal(Vector2.UnitY, transformToObj);

            Vector2 yPosInObj = Vector2.Transform(Vector2.Zero, transformToObj);

            for (int yA = 0; yA < this.height; yA++) {
                Vector2 posInObj = yPosInObj;

                for (int xA = 0; xA < this.width; xA++) {
                    int xB = (int)Math.Round(posInObj.X);
                    int yB = (int)Math.Round(posInObj.Y);

                    if (xB >= 0 && xB < obj.width &&
                        yB >= 0 && yB < obj.height) {
                        Color colA = this.bitmapData[xA + yA * this.width];
                        Color colB = obj.bitmapData[xB + yB * obj.width];

                        // If overlaping pixels are not transparent
                        if (colA.A != 0 && colB.A != 0) {
                            return true;
                        }
                    }
                    // Move to the next pixel in row direction
                    posInObj += stepX;
                }
                // Move to the next pixel in col direction
                yPosInObj += stepY;
            }
            return false;
        }

        public void UpdateOrigin()
        {
            if (this.bitmap != null) {
                origin = new Vector2(x + bitmap.Width / 2, y + bitmap.Height / 2);
            }           
        }
        public abstract void Update();
        public abstract void Draw(SpriteBatch spriteBatch, Camera C);
    }
}
