﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;

namespace ProjektStatki
{
    public class Player : Character
    {
        private List<Cannon> cannons;
        private Cannon activeCannon;
        public Cannon ActiveCannon { get => activeCannon; }

        public Player()
        {
            this.bitmap = Game1.playerImage;
            this.bitmapData = new Color[this.bitmap.Width * this.bitmap.Height];
            this.bitmap.GetData<Color>(this.bitmapData);
            this.xPos = 40;
            this.yPos = 40;
            this.hitbox = new Rectangle((int)xPos, (int)yPos, this.bitmap.Width, this.bitmap.Height);
            this.Health = 100;
            this.speed = 320;

            this.cannons = new List<Cannon>();
            this.AddCannon(new BasicCannon(this));

            // Subscribe self to game objects
            Game1.gObjects.Add(this);
            Game1.ships.Add(this);
        }

        public override void Draw(SpriteBatch spriteBatch, Camera C)
        {
            spriteBatch.Begin();
            spriteBatch.Draw(this.bitmap, new Vector2(this.x - C.xView, this.y - C.yView), null, Color.White, this.rotation, new Vector2(this.width / 2, this.height / 2), 1, SpriteEffects.None, 0);
            spriteBatch.End();
        }

        public override void Update()
        {
            xVelocity = Game1.inputMove.X;
            yVelocity = Game1.inputMove.Y;
            if (xVelocity != 0 || yVelocity != 0) { this.Move(); }
            this.HandleCannnon();

            this.xTile = this.x / 64;
            this.yTile = this.y / 64;

            // check objects collisions
            this.HandleObjectsCollision();

            // don't let player leave the world
            if (!this.hitbox.Within(Game1.worldRectangle))
            {
                if (this.x - this.width / 2 < 0) { this.xPos = this.width / 2; }
                if (this.y - this.height / 2 < 0) { this.yPos = this.height / 2; }
                if (this.x + this.width / 2 > Game1.worldWidth) { this.xPos = Game1.worldWidth - this.width / 2; }
                if (this.y + this.height / 2 > Game1.worldHeight) { this.yPos = Game1.worldHeight - this.height / 2; }
                this.hitbox.Set((int)xPos, (int)yPos);
            }
        }

        protected override void HandleObjectsCollision()
        {
            foreach (Map.Chunk tile in Game1.Level.Tiles) {
                int tileDist = Math.Abs(this.x / 64 - tile.col);
                    tileDist += Math.Abs(this.y / 64 - tile.row);
                if (tile.collidable && tileDist < 2 && this.Overlaps(tile)) {
                    this.MoveBack();
                }
            }
        }

        private void Move()
        {
            if (!Double.IsNaN(xVelocity)) { this.xPos += xVelocity * Game1.STEP * this.speed; }
            if (!Double.IsNaN(yVelocity)) { this.yPos += yVelocity * Game1.STEP * this.speed; }
            this.rotation = Game1.VectorToAngle(Game1.inputMove) + MathHelper.Pi;
           
            this.hitbox.Set((int)xPos, (int)yPos);
        }

        private void MoveBack()
        {
            if (!Double.IsNaN(xVelocity)) { this.xPos -= xVelocity * Game1.STEP * this.speed; }
            if (!Double.IsNaN(yVelocity)) { this.yPos -= yVelocity * Game1.STEP * this.speed; }
            this.rotation = Game1.VectorToAngle(Game1.inputMove) + MathHelper.Pi;

            this.hitbox.Set((int)xPos, (int)yPos);
        }

        private void AddCannon(Cannon cannon)
        {
            this.cannons.Add(cannon);
            this.activeCannon = cannon;
        }

        protected override void HandleCannnon()
        {
            if (this.activeCannon.cooldown > 0)
            {
                this.activeCannon.cooldown -= 1;
            }
        }
    }
}
